@@ -5,11 +5,11 @@ class Configure(config.package.CMakePackage):
     import os
     config.package.CMakePackage.__init__(self, framework)
     self.download          = ['git://https://github.com/meshadaptation/pragmatic.git']
-    self.gitcommit         = 'f56251de6ff39680d4ead2140b635e16fa6d604e'
+    self.gitcommit         = '1dfe81ff5f34b16c15ee61adaaa7f4974e5b5135'
     self.functions         = ['pragmatic_2d_init']
-    self.includes          = ['pragmatic.h']
+    self.includes          = ['pragmatic/pragmatic.h']
     self.liblist           = [['libpragmatic.a']]
-    self.includedir        = os.path.join('include', 'pragmatic')
+    self.requirescxx11     = 1
     return
 
   def setupDependencies(self, framework):
@@ -18,10 +18,11 @@ class Configure(config.package.CMakePackage):
     self.sharedLibraries = framework.require('PETSc.options.sharedLibraries', self)
     self.scalartypes     = framework.require('PETSc.options.scalarTypes',self)
     self.indexTypes      = framework.require('PETSc.options.indexTypes', self)
+    self.mpi             = framework.require('config.packages.MPI',self)
     self.metis           = framework.require('config.packages.metis', self)
     self.eigen           = framework.require('config.packages.eigen', self)
     self.mathlib         = framework.require('config.packages.mathlib',self)
-    self.deps            = [self.metis, self.eigen, self.mathlib]
+    self.deps            = [self.mpi, self.metis, self.eigen, self.mathlib]
     return
 
   def formCMakeConfigureArgs(self):
@@ -29,10 +30,23 @@ class Configure(config.package.CMakePackage):
       raise RuntimeError('CMake > 2.5 is needed to build Pragmatic')
 
     args = config.package.CMakePackage.formCMakeConfigureArgs(self)
-    args.append('-DMETIS_DIR='+self.metis.getInstallDir())
+    args.append('-DMETIS_DIR='+self.metis.directory)
     args.append('-DENABLE_VTK=OFF')
     args.append('-DENABLE_OPENMP=OFF')
     args.append('-DEIGEN_INCLUDE_DIR='+self.eigen.include[0])
+
+    # prevent Pragmatic from linking to MPI it finds by itself
+    args.append('-DMPI_C_COMPILER:STRING="'+self.framework.getCompiler()+'"')
+    args.append('-DMPI_C_INCLUDE_PATH:STRING=""')
+    args.append('-DMPI_C_COMPILE_FLAGS:STRING=""')
+    args.append('-DMPI_C_LINK_FLAGS:STRING=""')
+    args.append('-DMPI_C_LIBRARIES:STRING=""')
+    args.append('-DMPI_CXX_COMPILER:STRING="'+self.framework.getCompiler('Cxx')+'"')
+    args.append('-DMPI_CXX_INCLUDE_PATH:STRING=""')
+    args.append('-DMPI_CXX_COMPILE_FLAGS:STRING=""')
+    args.append('-DMPI_CXX_LINK_FLAGS:STRING=""')
+    args.append('-DMPI_CXX_LIBRARIES:STRING=""')
+
     if not self.compilerFlags.debugging:
       args.append('-DCMAKE_BUILD_TYPE=Release')
     if self.checkSharedLibrariesEnabled():
