@@ -1,6 +1,7 @@
 static char help[] = "Tests various DMPlex routines to construct, refine and distribute a mesh.\n\n";
 
 #include <petscdmplex.h>
+#include <petscsf.h>
 
 typedef enum {BOX, CYLINDER, SPHERE, BALL} DomainShape;
 enum {STAGE_LOAD, STAGE_DISTRIBUTE, STAGE_REFINE, STAGE_OVERLAP};
@@ -162,6 +163,7 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
                                          68, 69, 70, 82, 83, 84, 85, 102, 103, 105, 106, 107, 108, 109, 110, 111, 112, 114, 130, 132, 134, 135, 136, 137, 139};
   size_t         len, bdlen, extlen;
   PetscMPIInt    rank, size;
+  PetscBool      periodic;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -245,7 +247,18 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
     default: SETERRQ1(comm, PETSC_ERR_ARG_WRONG, "Unknown domain shape %D", user->domainShape);
     }
   }
-  ierr = DMLocalizeCoordinates(*dm);CHKERRQ(ierr); /* needed for periodic */
+
+  /* For topologically periodic meshes, we first localize coordinates,
+     and then remove any information related with the
+     automatic computation of localized vertices.
+     This way, refinement operations and conversions to p4est
+     will preserve the shape of the domain in physical space */
+  ierr = DMLocalizeCoordinates(*dm);CHKERRQ(ierr);
+  ierr = DMGetPeriodicity(*dm,&periodic,NULL,NULL,NULL);CHKERRQ(ierr);
+  if (periodic) {
+    ierr = DMSetPeriodicity(*dm,PETSC_TRUE,NULL,NULL,NULL);CHKERRQ(ierr);
+  }
+
   ierr = DMViewFromOptions(*dm,NULL,"-init_dm_view");CHKERRQ(ierr);
   ierr = DMGetDimension(*dm,&dim);CHKERRQ(ierr);
 
@@ -412,12 +425,18 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
 
   /* test redistribution of an already distributed mesh */
   if (user->redistribute) {
-    DM distributedMesh;
-
+    DM       distributedMesh;
+    PetscSF  sf;
+    PetscInt nranks;
 
     ierr = DMViewFromOptions(*dm, NULL, "-dm_pre_redist_view");CHKERRQ(ierr);
     ierr = DMPlexDistribute(*dm, 0, NULL, &distributedMesh);CHKERRQ(ierr);
     if (distributedMesh) {
+      ierr = DMGetPointSF(distributedMesh, &sf);CHKERRQ(ierr);
+      ierr = PetscSFSetUp(sf);CHKERRQ(ierr);
+      ierr = DMGetNeighbors(distributedMesh, &nranks, NULL);CHKERRQ(ierr);
+      ierr = MPI_Allreduce(MPI_IN_PLACE, &nranks, 1, MPIU_INT, MPI_MIN, PetscObjectComm((PetscObject)*dm));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)*dm)), "Minimum number of neighbors: %D\n", nranks);CHKERRQ(ierr);
       ierr = DMDestroy(dm);CHKERRQ(ierr);
       *dm  = distributedMesh;
     }
