@@ -5,6 +5,11 @@
 #include <petsc/private/petscimpl.h>        /*I  "petscsys.h"   I*/
 #include <petscvalgrind.h>
 #include <petscviewer.h>
+#if defined(PETSC_USE_GCOV)
+EXTERN_C_BEGIN
+void  __gcov_flush(void);
+EXTERN_C_END
+#endif
 
 #if defined(PETSC_USE_LOG)
 PETSC_INTERN PetscErrorCode PetscLogFinalize(void);
@@ -123,7 +128,7 @@ PetscErrorCode  PetscInitializeNoArguments(void)
 
 .seealso: PetscInitialize(), PetscInitializeNoArguments(), PetscInitializeFortran()
 @*/
-PetscErrorCode PetscInitialized(PetscBool  *isInitialized)
+PetscErrorCode PetscInitialized(PetscBool *isInitialized)
 {
   *isInitialized = PetscInitializeCalled;
   return 0;
@@ -773,7 +778,7 @@ PetscErrorCode  PetscInitialize(int *argc,char ***args,const char file[],const c
       https://wiki.mpich.org/mpich/index.php/ABI_Compatibility_Initiative which started with
         MPICH v3.1 (Released Feburary 2014)
         IBM MPI v2.1 (December 2014)
-        Intel® MPI Library v5.0 (2014)
+        Intel MPI Library v5.0 (2014)
         Cray MPT v7.0.0 (June 2014)
       As of July 31, 2017 the ABI number still appears to be 12, that is all of the versions
       listed above and since that time are compatible.
@@ -1611,6 +1616,13 @@ PetscErrorCode  PetscFinalize(void)
 
   PetscInitializeCalled = PETSC_FALSE;
   PetscFinalizeCalled   = PETSC_TRUE;
+#if defined(PETSC_USE_GCOV)
+  /*
+     flush gcov, otherwise during CI the flushing continues into the next pipeline resulting in git not being able to delete directories since the
+     gcov files are still being added to the directories as git tries to remove the directories.
+   */
+  __gcov_flush();
+#endif
   PetscFunctionReturn(0);
 }
 
