@@ -128,12 +128,14 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
 #if defined(PETSC_USE_REAL_SINGLE)
   tao->gatol       = 1e-5;
   tao->grtol       = 1e-5;
+  tao->crtol       = 1e-5;
+  tao->catol       = 1e-5;
 #else
   tao->gatol       = 1e-8;
   tao->grtol       = 1e-8;
+  tao->crtol       = 1e-8;
+  tao->catol       = 1e-8;
 #endif
-  tao->crtol       = 0.0;
-  tao->catol       = 0.0;
   tao->gttol       = 0.0;
   tao->steptol     = 0.0;
   tao->trust0      = PETSC_INFINITY;
@@ -155,6 +157,7 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
   tao->viewconstraints = PETSC_FALSE;
 
   tao->bounded = PETSC_FALSE;
+  tao->constrained = PETSC_FALSE;
 
   tao->header_printed = PETSC_FALSE;
 
@@ -648,7 +651,7 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer," gttol=%g\n",(double)tao->gttol);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"Residual in Function/Gradient:=%g\n",(double)tao->residual);CHKERRQ(ierr);
 
-    if (tao->cnorm>0 || tao->catol>0 || tao->crtol>0){
+    if (tao->constrained){
       ierr=PetscViewerASCIIPrintf(viewer,"convergence tolerances:");CHKERRQ(ierr);
       ierr=PetscViewerASCIIPrintf(viewer," catol=%g,",(double)tao->catol);CHKERRQ(ierr);
       ierr=PetscViewerASCIIPrintf(viewer," crtol=%g\n",(double)tao->crtol);CHKERRQ(ierr);
