@@ -1,16 +1,14 @@
 
-#include <private/vecimpl.h>      /*I "petscvec.h" I*/
+#include <petsc/private/vecimpl.h>           /*I  "petscvec.h"   I*/
 
-#undef __FUNCT__  
-#define __FUNCT__ "VecCreate"
 /*@
   VecCreate - Creates an empty vector object. The type can then be set with VecSetType(),
   or VecSetFromOptions().
 
-   If you never  call VecSetType() or VecSetFromOptions() it will generate an 
+   If you never  call VecSetType() or VecSetFromOptions() it will generate an
    error when you try to use the vector.
 
-  Collective on MPI_Comm
+  Collective
 
   Input Parameter:
 . comm - The communicator for the vector object
@@ -20,7 +18,6 @@
 
   Level: beginner
 
-.keywords: vector, create
 .seealso: VecSetType(), VecSetSizes(), VecCreateMPIWithArray(), VecCreateMPI(), VecDuplicate(),
           VecDuplicateVecs(), VecCreateGhost(), VecCreateSeq(), VecPlaceArray()
 @*/
@@ -31,19 +28,20 @@ PetscErrorCode  VecCreate(MPI_Comm comm, Vec *vec)
 
   PetscFunctionBegin;
   PetscValidPointer(vec,2);
-  *vec = PETSC_NULL;
-#ifndef PETSC_USE_DYNAMIC_LIBRARIES
-  ierr = VecInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-#endif
+  *vec = NULL;
+  ierr = VecInitializePackage();CHKERRQ(ierr);
 
-  ierr = PetscHeaderCreate(v, _p_Vec, struct _VecOps, VEC_CLASSID, -1, "Vec", "Vector", "Vec", comm, VecDestroy, VecView);CHKERRQ(ierr);
-  ierr = PetscMemzero(v->ops, sizeof(struct _VecOps));CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(v, VEC_CLASSID, "Vec", "Vector", "Vec", comm, VecDestroy, VecView);CHKERRQ(ierr);
 
   ierr            = PetscLayoutCreate(comm,&v->map);CHKERRQ(ierr);
   v->array_gotten = PETSC_FALSE;
   v->petscnative  = PETSC_FALSE;
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_CUDA)
+  v->minimum_bytes_pinned_memory = 0;
+  v->pinned_memory = PETSC_FALSE;
+#endif
 
-  *vec = v; 
+  *vec = v;
   PetscFunctionReturn(0);
 }
 
